@@ -1,8 +1,6 @@
 from .box import Box
 
-
-class UnsupportedBox(RuntimeError):
-    pass
+from .error import UnsupportedBoxResponse
 
 
 class Products:
@@ -121,13 +119,22 @@ class Products:
                     ]
                 ],
             },
-            "switchBox": {
+            "switchBox0": {
                 "api_path": "/api/relay/state",
                 "api_level_range": [20180604, 20190808],
                 "api": {
                     "on": lambda x=None: ("GET", "/s/1", None),
                     "off": lambda x=None: ("GET", "/s/0", None),
                 },
+                "switches": [["0.relay", {"state": "[relay=0]/state"}, "relay"]],
+            },
+            "switchBox": {
+                "api_path": "/api/relay/state",
+                "api_level_range": [20190808, 20190808],
+                "api": {
+                    "on": lambda x=None: ("GET", "/s/1", None),
+                    "off": lambda x=None: ("GET", "/s/0", None),
+                },
                 "switches": [["0.relay", {"state": "relays/[relay=0]/state"}, "relay"]],
             },
             "switchBoxD": {
@@ -206,8 +213,8 @@ class Products:
             data = {
                 "switchBox": [info],
                 "dimmerBox": [info, root],
-                "wLightBoxS": [info, root],
-                "wLightBox": [info, root],
+                "wLightBoxS": [info],
+                "wLightBox": [info],
                 "gateController": [info],
                 "saunaBox": [info],
                 "switchBoxD": [info],
@@ -227,11 +234,11 @@ class Products:
         try:
             product_type = info["type"]
         except KeyError:
-            raise UnsupportedBox("(no type in info)")
+            raise UnsupportedBoxResponse("Missing 'type' field:", info)
 
         try:
             args = data[product_type]
         except KeyError:
-            raise UnsupportedBox(product_type)
+            raise UnsupportedBoxResponse(product_type)
 
         return Box(api_host, *args)
