@@ -1,6 +1,10 @@
 """BleBox cover entities tests."""
 import json
 
+import pytest
+
+from blebox_uniapi import error
+
 from .conftest import DefaultBoxTest, jmerge, CommonEntity
 
 # TODO: remove
@@ -153,9 +157,7 @@ class TestShutter(CoverTest):
 
     def patch_version(apiLevel):
         """Generate a patch for a JSON state fixture."""
-        return f"""
-        {{ "device": {{ "apiLevel": {apiLevel} }} }}
-        """
+        return f"""{{ "device": {{ "apiLevel": {apiLevel} }} }}"""
 
     DEVICE_INFO_FUTURE = jmerge(DEVICE_INFO, patch_version(20190912))
     DEVICE_INFO_LATEST = jmerge(DEVICE_INFO, patch_version(20190911))
@@ -164,6 +166,21 @@ class TestShutter(CoverTest):
     DEVICE_INFO_MINIMUM = jmerge(DEVICE_INFO, patch_version(20180604))
     DEVICE_INFO_UNSUPPORTED = jmerge(DEVICE_INFO, patch_version(20180603))
 
+    DEVICE_INFO_UNSPECIFIED_API = json.loads(
+        """
+    {
+        "device": {
+            "deviceName": "My shutter 1",
+            "type": "shutterBox",
+            "fv": "0.147",
+            "hv": "0.7",
+            "id": "2bee34e750b8",
+            "ip": "172.0.0.1"
+        }
+    }
+    """
+    )
+
     STATE_DEFAULT = json.loads(
         """
     {
@@ -203,6 +220,7 @@ class TestShutter(CoverTest):
     STATE_OPENING = jmerge(STATE_DEFAULT, patch_state(1, 34, 0))
     STATE_MINIMALLY_OPENING = jmerge(STATE_DEFAULT, patch_state(1, 97, 100))
     STATE_STOPPED = jmerge(STATE_DEFAULT, patch_state(2, 34, 100))
+    STATE_UNKNOWN = jmerge(STATE_DEFAULT, patch_state(2, 34, -1))
 
     async def test_init(self, aioclient_mock):
         """Test cover default state."""
@@ -275,12 +293,18 @@ class TestShutter(CoverTest):
         await entity.async_stop_cover()
         self.assert_state(entity, STATE_OPEN)
 
+    async def test_unkown_position(self, aioclient_mock):
+        """Test handling cover at unknown position."""
+        entity = await self.updated(aioclient_mock, self.STATE_UNKNOWN)
+        self.assert_state(entity, STATE_OPEN)
+
 
 class TestGateBox(CoverTest):
     """Tests for cover devices representing a BleBox gateBox."""
 
     DEV_INFO_PATH = "api/gate/state"
 
+    # TODO: does gateBox have an api level currently?
     DEVICE_INFO = json.loads(
         """
         {
@@ -294,18 +318,14 @@ class TestGateBox(CoverTest):
         """
     )
 
-    def patch_version(apiLevel):
-        """Generate a patch for a JSON state fixture."""
-        return f"""
-        {{ "device": {{ "apiLevel": {apiLevel} }} }}
-        """
-
     DEVICE_INFO_FUTURE = DEVICE_INFO
     DEVICE_INFO_LATEST = DEVICE_INFO
     DEVICE_INFO_OUTDATED = DEVICE_INFO
     DEVICE_INFO_MINIMUM = DEVICE_INFO
     DEVICE_INFO_UNSUPPORTED = DEVICE_INFO
 
+    DEVICE_INFO_UNSPECIFIED_API = None  # already handled as default case
+
     STATE_DEFAULT = json.loads(
         """
         {
@@ -330,6 +350,9 @@ class TestGateBox(CoverTest):
     STATE_OPENING = jmerge(STATE_DEFAULT, '{ "currentPos": 50, "desiredPos": 100 }')
     STATE_CLOSING = jmerge(STATE_DEFAULT, '{ "currentPos": 50, "desiredPos": 0 }')
     STATE_STOPPED = jmerge(STATE_DEFAULT, '{ "currentPos": 50, "desiredPos": 50 }')
+    STATE_FULLY_OPENED = jmerge(
+        STATE_DEFAULT, '{ "currentPos": 100, "desiredPos": 100 }'
+    )
 
     STATE_OPENING_NO_STOP = jmerge(STATE_OPENING, '{ "extraButtonType": 3}')
 
@@ -370,6 +393,10 @@ class TestGateBox(CoverTest):
         await entity.async_open_cover()
         self.assert_state(entity, STATE_OPENING)
 
+    async def test_fully_opened(self, aioclient_mock):
+        entity = await self.updated(aioclient_mock, self.STATE_FULLY_OPENED)
+        assert entity.state == STATE_OPEN
+
     async def test_close(self, aioclient_mock):
         """Test cover closing."""
 
@@ -379,6 +406,12 @@ class TestGateBox(CoverTest):
         await entity.async_close_cover()
         self.assert_state(entity, STATE_CLOSING)
 
+    async def test_closed(self, aioclient_mock):
+        """Test cover closed state."""
+
+        entity = await self.updated(aioclient_mock, self.STATE_CLOSED)
+        self.assert_state(entity, STATE_CLOSED)
+
     async def test_stop(self, aioclient_mock):
         """Test cover stopping."""
 
@@ -400,6 +433,23 @@ class TestGateBox(CoverTest):
         entity = await self.updated(aioclient_mock, self.STATE_OPENING_NO_STOP)
         assert not entity.supported_features & SUPPORT_STOP
 
+    async def test_stop_with_no_stop(self, aioclient_mock):
+        """Test stop capability is not available."""
+
+        entity = await self.updated(aioclient_mock, self.STATE_OPENING_NO_STOP)
+
+        with pytest.raises(
+            error.MisconfiguredDevice, match=r"second button not configured as 'stop'"
+        ):
+            await entity.async_stop_cover()
+
+    async def test_set_position(self, aioclient_mock):
+        """Test cover position setting."""
+
+        entity = await self.updated(aioclient_mock, self.STATE_CLOSED)
+        with pytest.raises(NotImplementedError):
+            await entity.async_set_cover_position(**{ATTR_POSITION: 1})  # almost closed
+
 
 class TestGateController(CoverTest):
     """Tests for cover devices representing a BleBox gateController."""
@@ -435,6 +485,22 @@ class TestGateController(CoverTest):
     DEVICE_INFO_MINIMUM = jmerge(DEVICE_INFO, patch_version(20180604))
     DEVICE_INFO_UNSUPPORTED = jmerge(DEVICE_INFO, patch_version(20180603))
 
+    # NOTE: can't happen with a real device
+    DEVICE_INFO_UNSPECIFIED_API = json.loads(
+        """
+    {
+      "device": {
+        "deviceName": "My gate controller 1",
+        "type": "gateController",
+        "fv": "1.390",
+        "hv": "custom.2.6",
+        "id": "0ff2ffaafe30db9437",
+        "ip": "192.168.1.11"
+      }
+    }
+    """
+    )
+
     STATE_DEFAULT = json.loads(
         """
     {
