@@ -3,9 +3,13 @@ import json
 
 from .conftest import DefaultBoxTest, jmerge, CommonEntity
 
+from blebox_uniapi.error import UnsupportedBoxResponse, BadOnValueError
+
 # TODO: remove
 import colorsys
 
+import pytest
+
 # TODO: remove
 ATTR_BRIGHTNESS = "brightness"
 ATTR_HS_COLOR = "ATTR_HS_COLOR"
@@ -62,10 +66,6 @@ def color_rgb_to_hex(r: int, g: int, b: int) -> str:
     return "{0:02x}{1:02x}{2:02x}".format(round(r), round(g), round(b))
 
 
-class BadOnValueError(RuntimeError):
-    pass
-
-
 class BleBoxLightEntity(CommonEntity):
     """Representation of BleBox lights."""
 
@@ -96,6 +96,9 @@ class BleBoxLightEntity(CommonEntity):
     def hs_color(self):
         """Return the hue and saturation."""
         rgbw_hex = self._feature.rgbw_hex
+        if rgbw_hex is None:
+            return None
+
         r, g, b, x = rgb_hex_to_rgb_list(rgbw_hex)
         return color_RGB_to_hs(r, g, b)
 
@@ -178,6 +181,35 @@ class TestDimmer(DefaultBoxTest):
     DEVICE_INFO_MINIMUM = jmerge(DEVICE_INFO, patch_version(20170829))
     DEVICE_INFO_UNSUPPORTED = jmerge(DEVICE_INFO, patch_version(20170828))
 
+    DEVICE_INFO_UNSPECIFIED_API = json.loads(
+        """
+    {
+        "device": {
+            "deviceName": "My dimmer",
+            "type": "dimmerBox",
+            "fv": "0.247",
+            "hv": "0.2",
+            "id": "1afe34e750b8"
+        },
+        "network": {
+            "ip": "192.168.1.239",
+            "ssid": "myWiFiNetwork",
+            "station_status": 5,
+            "apSSID": "dimmerBox-ap",
+            "apPasswd": ""
+        },
+        "dimmer": {
+            "loadType": 7,
+            "currentBrightness": 65,
+            "desiredBrightness": 65,
+            "temperature": 39,
+            "overloaded": false,
+            "overheated": false
+        }
+    }
+    """
+    )
+
     STATE_DEFAULT = json.loads(
         """
     {
@@ -297,22 +329,13 @@ class TestWLightBoxS(DefaultBoxTest):
         "device": {
             "deviceName": "My wLightBoxS",
             "type": "wLightBoxS",
-            "fv": "0.247",
-            "hv": "0.2",
+            "fv": "0.924",
+            "hv": "0.1",
+            "universe": 0,
             "id": "1afe34e750b8",
-            "apiLevel": 20180718
-        },
-        "network": {
-            "ip": "192.168.1.239",
-            "ssid": "myWiFiNetwork",
-            "station_status": 5,
-            "apSSID": "wLightBoxS-ap",
-            "apPasswd": ""
-        },
-        "light": {
-            "desiredColor": "E3",
-            "currentColor": "DF",
-            "fadeSpeed": 255
+            "apiLevel": 20180718,
+            "ip": "192.168.9.13",
+            "availableFv": null
         }
     }
     """
@@ -331,12 +354,50 @@ class TestWLightBoxS(DefaultBoxTest):
     DEVICE_INFO_MINIMUM = jmerge(DEVICE_INFO, patch_version(20180718))
     DEVICE_INFO_UNSUPPORTED = jmerge(DEVICE_INFO, patch_version(20180717))
 
+    DEVICE_INFO_UNSPECIFIED_API = json.loads(
+        """
+    {
+        "device": {
+            "deviceName": "My wLightBoxS",
+            "type": "wLightBoxS",
+            "fv": "0.247",
+            "hv": "0.2",
+            "id": "1afe34e750b8"
+        },
+        "network": {
+            "ip": "192.168.1.239",
+            "ssid": "myWiFiNetwork",
+            "station_status": 5,
+            "apSSID": "wLightBoxS-ap",
+            "apPasswd": ""
+        },
+        "light": {
+            "desiredColor": "e3",
+            "currentColor": "df",
+            "fadeSpeed": 255
+        }
+    }
+    """
+    )
+
     STATE_ON = json.loads(
         """
         {
             "light": {
-                "desiredColor": "AB",
-                "currentColor": "CD",
+                "desiredColor": "ab",
+                "currentColor": "cd",
+                "fadeSpeed": 255
+            }
+        }
+    """
+    )
+
+    STATE_FULL_ON = json.loads(
+        """
+        {
+            "light": {
+                "desiredColor": "ff",
+                "currentColor": "ce",
                 "fadeSpeed": 255
             }
         }
@@ -366,9 +427,9 @@ class TestWLightBoxS(DefaultBoxTest):
         assert entity.unique_id == "BleBox-wLightBoxS-1afe34e750b8-color"
 
         assert entity.supported_features & SUPPORT_BRIGHTNESS
-        assert entity.brightness == 0xE3
+        assert entity.brightness is None
 
-        assert entity.is_on is True  # state already available
+        assert entity.is_on is None
 
     async def test_update(self, aioclient_mock):
         """Test light updating."""
@@ -398,7 +459,40 @@ class TestWLightBoxS(DefaultBoxTest):
         async def turn_on():
             await entity.async_turn_on()
 
-        # TODO: implement last-value-after-on?
+        await self.allow_set_brightness(
+            turn_on, aioclient_mock, 0xFF, self.STATE_FULL_ON
+        )
+
+        assert entity.is_on is True
+        assert entity.brightness == 0xFF
+
+    async def test_on_with_bad_value_type(self, aioclient_mock):
+        """Test light on with off value."""
+        entity = await self.updated(aioclient_mock, self.STATE_OFF)
+        assert entity.is_on is False
+
+        with pytest.raises(
+            BadOnValueError,
+            match=r"adjust_brightness called with bad parameter \(00 is <class 'str'> instead of int\)",
+        ):
+            await entity.async_turn_on(brightness="00")
+
+    async def test_on_with_bad_value_exceeding_max(self, aioclient_mock):
+        """Test light on with off value."""
+        entity = await self.updated(aioclient_mock, self.STATE_OFF)
+        assert entity.is_on is False
+
+        with pytest.raises(
+            BadOnValueError,
+            match=r"adjust_brightness called with bad parameter \(1234 is greater than 255\)",
+        ):
+            await entity.async_turn_on(brightness=1234)
+
+    async def test_off(self, aioclient_mock):
+        """Test light off."""
+        entity = await self.updated(aioclient_mock, self.STATE_ON)
+        assert entity.is_on is True
+
         async def turn_off():
             await entity.async_turn_off()
 
@@ -423,10 +517,37 @@ class TestWLightBox(DefaultBoxTest):
         "device": {
             "deviceName": "My light 1",
             "type": "wLightBox",
-            "fv": "0.247",
-            "hv": "0.2",
+            "fv": "0.993",
+            "hv": "4.3",
             "id": "1afe34e750b8",
             "apiLevel": 20190808
+        }
+    }
+    """
+    )
+
+    def patch_version(apiLevel):
+        """Generate a patch for a JSON state fixture."""
+        return f"""
+        {{ "device": {{ "apiLevel": {apiLevel} }} }}
+        """
+
+    DEVICE_INFO_FUTURE = jmerge(DEVICE_INFO, patch_version(20190809))
+    DEVICE_INFO_LATEST = jmerge(DEVICE_INFO, patch_version(20190808))
+    DEVICE_INFO_OUTDATED = jmerge(DEVICE_INFO, patch_version(20190807))
+
+    DEVICE_INFO_MINIMUM = jmerge(DEVICE_INFO, patch_version(20180718))
+    DEVICE_INFO_UNSUPPORTED = jmerge(DEVICE_INFO, patch_version(20180717))
+
+    DEVICE_INFO_UNSPECIFIED_API = json.loads(
+        """
+    {
+        "device": {
+            "deviceName": "My light 1",
+            "type": "wLightBox",
+            "fv": "0.247",
+            "hv": "0.2",
+            "id": "1afe34e750b8"
         },
         "network": {
             "ip": "192.168.1.237",
@@ -447,19 +568,6 @@ class TestWLightBox(DefaultBoxTest):
             """
     )
 
-    def patch_version(apiLevel):
-        """Generate a patch for a JSON state fixture."""
-        return f"""
-        {{ "device": {{ "apiLevel": {apiLevel} }} }}
-        """
-
-    DEVICE_INFO_FUTURE = jmerge(DEVICE_INFO, patch_version(20190809))
-    DEVICE_INFO_LATEST = jmerge(DEVICE_INFO, patch_version(20190808))
-    DEVICE_INFO_OUTDATED = jmerge(DEVICE_INFO, patch_version(20190807))
-
-    DEVICE_INFO_MINIMUM = jmerge(DEVICE_INFO, patch_version(20180718))
-    DEVICE_INFO_UNSUPPORTED = jmerge(DEVICE_INFO, patch_version(20180717))
-
     STATE_DEFAULT = json.loads(
         """
     {
@@ -520,15 +628,15 @@ class TestWLightBox(DefaultBoxTest):
         assert entity.unique_id == "BleBox-wLightBox-1afe34e750b8-color"
 
         assert entity.supported_features & SUPPORT_WHITE_VALUE
-        assert entity.white_value == 0xD9
+        assert entity.white_value is None
 
         assert entity.supported_features & SUPPORT_COLOR
-        assert entity.hs_color == (210.0, 28.452)
-        assert entity.white_value == 0xD9
+        assert entity.hs_color is None
+        assert entity.white_value is None
 
         # assert entity.supported_features & SUPPORT_BRIGHTNESS
         # assert entity.brightness == 123
-        assert entity.is_on is True  # state already available
+        assert entity.is_on is None
 
     async def test_update(self, aioclient_mock):
         """Test light updating."""
@@ -652,3 +760,24 @@ class TestWLightBox(DefaultBoxTest):
         assert entity.is_on is False
         assert entity.hs_color == (0, 0)
         assert entity.white_value == 0x00
+
+    async def test_ancient_response(self, aioclient_mock):
+        """Test e.g. unsupported, ancient device status structure."""
+
+        DEVICE_INFO_ANCIENT_STRUCTURE = json.loads(
+            """
+            {
+                "deviceName": "My light 1",
+                "type": "wLightBox",
+                "fv": "0.623",
+                "hv": "0.3",
+                "universe": 0,
+                "id": "6201943ff9c9",
+                "ip": "192.168.9.45"
+            }
+            """
+        )
+
+        await self.allow_get_info(aioclient_mock, DEVICE_INFO_ANCIENT_STRUCTURE)
+        with pytest.raises(UnsupportedBoxResponse):
+            await self.async_entities(aioclient_mock)
