@@ -24,6 +24,118 @@ class BleBoxSwitchEntity(CommonEntity):
         return await self._feature.async_turn_off()
 
 
+class TestSwitchBox0(DefaultBoxTest):
+    """Tests for BleBox switchBox."""
+
+    DEVCLASS = "switches"
+    ENTITY_CLASS = BleBoxSwitchEntity
+
+    DEV_INFO_PATH = "api/relay/state"
+
+    DEVICE_INFO = json.loads(
+        """
+    {
+        "device":   {
+            "deviceName": "My switchBox",
+            "type": "switchBox",
+            "fv": "0.247",
+            "hv": "0.2",
+            "id": "1afe34e750b8",
+            "ip": "192.168.1.239",
+            "apiLevel": "20180604"
+        }
+    }
+    """
+    )
+
+    def patch_version(apiLevel):
+        """Generate a patch for a JSON state fixture."""
+        return f"""
+        {{ "device": {{ "apiLevel": {apiLevel} }} }}
+        """
+
+    # TODO: make future far off to make tests less brittle?
+    DEVICE_INFO_FUTURE = jmerge(DEVICE_INFO, patch_version(20190809))
+    DEVICE_INFO_LATEST = jmerge(DEVICE_INFO, patch_version(20190808))
+    DEVICE_INFO_OUTDATED = jmerge(DEVICE_INFO, patch_version(20190807))
+    DEVICE_INFO_MINIMUM = jmerge(DEVICE_INFO, patch_version(20180604))
+    DEVICE_INFO_UNSUPPORTED = jmerge(DEVICE_INFO, patch_version(20180603))
+
+    DEVICE_INFO_UNSPECIFIED_API = json.loads(
+        """
+    {
+        "device":   {
+            "deviceName": "My switchBox",
+            "type": "switchBox",
+            "fv": "0.247",
+            "hv": "0.2",
+            "id": "1afe34e750b8",
+            "ip": "192.168.1.239"
+        }
+    }
+    """
+    )
+
+    STATE_DEFAULT = json.loads(
+        """
+        [{
+            "relay": 0,
+            "state": 0,
+            "stateAfterRestart": 0
+        }]
+        """
+    )
+
+    def patch_state(state):
+        """Generate a patch for a JSON state fixture."""
+        return f'[ {{ "state": {state} }} ]'
+
+    STATE_OFF = STATE_DEFAULT
+    STATE_ON = jmerge(STATE_DEFAULT, patch_state(1))
+
+    async def test_init(self, aioclient_mock):
+        """Test switch default state."""
+
+        await self.allow_get_info(aioclient_mock)
+
+        entity = (await self.async_entities(aioclient_mock))[0]
+
+        assert entity.name == "switchBox0-0.relay"
+        assert entity.unique_id == "BleBox-switchBox0-1afe34e750b8-0.relay"
+
+        assert entity.device_class == DEVICE_CLASS_SWITCH
+
+        assert entity.is_on is None
+
+    async def test_update_when_off(self, aioclient_mock):
+        """Test switch updating when off."""
+
+        entity = await self.updated(aioclient_mock, self.STATE_OFF)
+        assert entity.is_on is False
+
+    async def test_update_when_on(self, aioclient_mock):
+        """Test switch updating when on."""
+
+        entity = await self.updated(aioclient_mock, self.STATE_ON)
+        assert entity.is_on is True
+
+    async def test_on(self, aioclient_mock):
+        """Test turning switch on."""
+
+        entity = await self.updated(aioclient_mock, self.STATE_OFF)
+        self.allow_get(aioclient_mock, "/s/1", self.STATE_ON)
+        await entity.async_turn_on()
+        assert entity.is_on is True
+
+    async def test_off(self, aioclient_mock):
+        """Test turning switch off."""
+
+        entity = await self.updated(aioclient_mock, self.STATE_ON)
+        self.allow_get(aioclient_mock, "/s/0", self.STATE_OFF)
+        await entity.async_turn_off()
+        assert entity.is_on is False
+
+
 class TestSwitchBox(DefaultBoxTest):
     """Tests for BleBox switchBox."""
 
@@ -55,11 +167,28 @@ class TestSwitchBox(DefaultBoxTest):
         """
 
     # TODO: make future far off to make tests less brittle?
-    DEVICE_INFO_FUTURE = jmerge(DEVICE_INFO, patch_version(20190809))
+    DEVICE_INFO_FUTURE = jmerge(DEVICE_INFO, patch_version(20190808 + 1))
     DEVICE_INFO_LATEST = jmerge(DEVICE_INFO, patch_version(20190808))
-    DEVICE_INFO_OUTDATED = jmerge(DEVICE_INFO, patch_version(20190807))
-    DEVICE_INFO_MINIMUM = jmerge(DEVICE_INFO, patch_version(20180604))
-    DEVICE_INFO_UNSUPPORTED = jmerge(DEVICE_INFO, patch_version(20180603))
+    DEVICE_INFO_OUTDATED = jmerge(DEVICE_INFO, patch_version(20190808))
+    DEVICE_INFO_MINIMUM = jmerge(DEVICE_INFO, patch_version(20190808))
+
+    # since below it 20180808 it switches to switchBox0
+    DEVICE_INFO_UNSUPPORTED = jmerge(DEVICE_INFO, patch_version(20180604 - 1))
+
+    DEVICE_INFO_UNSPECIFIED_API = json.loads(
+        """
+    {
+        "device":   {
+            "deviceName": "My switchBox",
+            "type": "switchBox",
+            "fv": "0.247",
+            "hv": "0.2",
+            "id": "1afe34e750b8",
+            "ip": "192.168.1.239"
+        }
+    }
+    """
+    )
 
     STATE_DEFAULT = json.loads(
         """
@@ -179,6 +308,20 @@ class TestSwitchBoxD(DefaultBoxTest):
     DEVICE_INFO_MINIMUM = jmerge(DEVICE_INFO, patch_version(20190808))
     DEVICE_INFO_UNSUPPORTED = jmerge(DEVICE_INFO, patch_version(20190807))
 
+    DEVICE_INFO_UNSPECIFIED_API = json.loads(
+        """
+    {
+        "device": {
+            "deviceName": "My switchBoxD",
+            "type": "switchBoxD",
+            "fv": "0.200",
+            "hv": "0.7",
+            "id": "1afe34e750b8"
+        }
+    }
+    """
+    )
+
     def patch_state(state1, state2):
         """Generate a patch for a JSON state fixture."""
         return f"""
